"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class FunctionMatchers {
    constructor(fn) {
        this.fn = fn;
    }
    execute() {
        try {
            return { thrown: false, result: this.fn() };
        }
        catch (error) {
            return { thrown: true, error };
        }
    }
    match(handleFailure, handleSuccess) {
        const report = this.execute();
        return report.thrown ? handleFailure(report.error) : handleSuccess(report.result);
    }
    get throws() {
        return this.match(x => expect(x), value => {
            throw new Error(`Expecting the matched function to throw an error, but it returns a value: ${value}`);
        });
    }
    get returns() {
        return this.match(error => {
            throw new Error(`Expecting the matched function to return a value, but it throws an error: ${error}`);
        }, x => expect(x));
    }
}
exports.FunctionMatchers = FunctionMatchers;
function expectFunction(fn) {
    return new FunctionMatchers(fn);
}
exports.expectFunction = expectFunction;
exports.default = expectFunction;
//# sourceMappingURL=expect-function.js.map