import * as fsTreeUtils from 'fs-tree-utils';
import Tree = fsTreeUtils.Tree;
export declare type PromiseFunc<X, Y> = (x: X) => Promise<Y>;
export declare type SyncFunc<X, Y> = (x: X) => Y;
export declare type SetupFunc<Y> = () => Promise<Y>;
export declare type TeardownFunc<X> = (x: X) => Promise<void>;
export declare type CalledFunc<X, Y> = PromiseFunc<X, Y>;
export declare type SyncCalledFunc<X, Y> = SyncFunc<X, Y>;
export declare type Tester = () => Promise<void>;
export declare type AsyncTesterFactory<SM, MT> = (fn: CalledFunc<SM, MT>) => Tester;
export declare type SyncTesterFactory<SM, MT> = (fn: SyncCalledFunc<SM, MT>) => Tester;
export interface Config<SY, TX> {
    readonly setup: SetupFunc<SY>;
    readonly teardown: TeardownFunc<TX>;
}
export declare type TesterFactory<SM, MT> = AsyncTesterFactory<SM, MT> & {
    forAsync: AsyncTesterFactory<SM, MT>;
    forSync: SyncTesterFactory<SM, MT>;
};
export declare namespace base {
    function createFactory<SM, MT>({ setup, teardown }: Config<SM, MT>): TesterFactory<SM, MT>;
}
export declare const createFactory: typeof base.createFactory;
export declare namespace virtualEnvironment {
    interface Info {
        readonly tree: Tree.Write.Node;
        readonly container: string;
        readonly previousWorkingDirectory: string;
    }
    function createFactory(tree: Tree.Write.Node, container?: string): {
        info: Info;
        apply: TesterFactory<Info, void>;
    };
}
export default createFactory;
