"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const process = tslib_1.__importStar(require("process"));
const fsx = tslib_1.__importStar(require("fs-extra"));
const fsTreeUtils = tslib_1.__importStar(require("fs-tree-utils"));
const unique_temp_path_1 = tslib_1.__importDefault(require("unique-temp-path"));
var base;
(function (base) {
    function createFactory({ setup, teardown }) {
        const forAsync = (fn) => async () => {
            const sm = await setup();
            const mt = await fn(sm);
            await teardown(mt);
        };
        const forSync = (fn) => forAsync(async (sm) => fn(sm));
        return Object.assign(forAsync, {
            forAsync,
            forSync
        });
    }
    base.createFactory = createFactory;
})(base = exports.base || (exports.base = {}));
exports.createFactory = base.createFactory;
var virtualEnvironment;
(function (virtualEnvironment) {
    function createFactory(tree, container = unique_temp_path_1.default()) {
        const previousWorkingDirectory = process.cwd();
        const info = { tree, container, previousWorkingDirectory };
        async function populate() {
            await depopulate();
            await fsTreeUtils.create(tree, container);
            return info;
        }
        async function depopulate() {
            await fsx.remove(container);
            return info;
        }
        const setup = async () => {
            await populate();
            process.chdir(container);
            return info;
        };
        const teardown = async () => {
            process.chdir(previousWorkingDirectory);
            await depopulate();
        };
        const apply = base.createFactory({ setup, teardown });
        return { info, apply };
    }
    virtualEnvironment.createFactory = createFactory;
})(virtualEnvironment = exports.virtualEnvironment || (exports.virtualEnvironment = {}));
exports.default = exports.createFactory;
//# sourceMappingURL=setup-teardown.js.map